#ifndef FLY_HOSTADDRESS
#define FLY_HOSTADDRESS

#include <QFrame>
#include <QHostAddress>

#include <fly/flyui_export.h>

class QLabel;
class QColor;

class MyLineEdit;

//********************************************************************
//* Represents IP address                                            *
//********************************************************************
//class FlyHostAddress: public QFrame
class FLYUI_EXPORT FlyHostAddress: public QFrame
{
    Q_OBJECT

    Q_PROPERTY(QString text1 READ first  WRITE setFirst)
    Q_PROPERTY(QString text2 READ second WRITE setSecond)
    Q_PROPERTY(QString text3 READ third  WRITE setThird)
    Q_PROPERTY(QString text4 READ fourth WRITE setFourth)
    Q_PROPERTY(QColor  paletteBackgroundColor READ paletteBackgroundColor WRITE setPaletteBackgroundColor)

protected:
    QHostAddress addr;
    MyLineEdit  *firstTriad,
                *secondTriad,
                *thirdTriad,
                *fourthTriad;
    QLabel       *firstDot,
                 *secondDot,
                 *thirdDot;

    friend class MyLineEdit;

    virtual void init();
public:
    FlyHostAddress(QWidget *parent);
    FlyHostAddress(const QHostAddress &address, QWidget *parent);
    ~FlyHostAddress();

    bool                 isEmpty()          const;
    bool                 isValid()          const;
    bool                 isReadOnly()       const;
    bool                 frame()            const;
    void                 setFrame(bool yes);

    QHostAddress         address() const;
    virtual       void   setPaletteBackgroundColor(const QColor &color);
    virtual const QColor &paletteBackgroundColor() const;

    // -----------------------------------------------
    // The following 8 functions are for internal use.
    // The are public due to Qt Designer's specifics
    // and SHOULD NOT be used by a user, though it's
    // not harmful.
    // They ARE NOT documented in a class reference.
    // -----------------------------------------------
    QString              first() const;
    QString              second() const;
    QString              third() const;
    QString              fourth() const;

    void                 setFirst(QString text);
    void                 setSecond(QString text);
    void                 setThird(QString text);
    void                 setFourth(QString text);
    // -----------------------------------------------

public slots:
    void                 setAddress(const QHostAddress &newAddress);
    void                 setAddress(const QString &newAddress);
    void                 clear();
    void                 setReadOnly(bool yes);
    void                 selectAll();
    void                 selectSection(int number);
    void                 deselect();

signals:
    void                 addressChanged(const QHostAddress &address);
    void                 returnPressed();

private slots:
    void                 emitAddressChanged();
};
#endif
