#ifndef FLYMENUVIEW_P_H
#define FLYMENUVIEW_P_H

#include <QTreeView>

#include <fly/flyui_export.h>

class FlyMenuModel;
class FlyMenuProxyModel;

class FLYUI_EXPORT FlyMenuView : public QTreeView
{
    Q_OBJECT
public:
    FlyMenuView(QWidget *parent = nullptr);
    FlyMenuView(const QString &menuPath, int iconSize, bool editable, QWidget *parent = nullptr);
    ~FlyMenuView();

    QMenu *contextMenu()
    {
        return editMenu;
    }

    QAction *newFileAction()
    {
        return newFileAct;
    }
    QAction *newDirAction()
    {
        return newDirAct;
    }
    QAction *delAction()
    {
        return delAct;
    }
    QAction *editAction()
    {
        return editAct;
    }
    QAction *exeAction()
    {
        return exeAct;
    }

    QAction *cutAction()
    {
        return cutAct;
    }
    QAction *copyAction()
    {
        return copyAct;
    }
    QAction *pasteAction()
    {
        return pasteAct;
    }

    QAction *upAction()
    {
        return upAct;
    }
    QAction *downAction()
    {
        return downAct;
    }
    QAction *recreateAction()
    {
        return recreateAct;
    }

    QModelIndex selectedItem();
    QString     selectedPath();

    void showAndSelect(const QString &path);
    bool createNew(bool dir);

    bool fixSortOrderForNew(const QString &newName, int beforeRow, const QModelIndex& parentIdx);
    void fixSortOrderInDir(const QModelIndex &idx, bool moveToUp);
    void invalidateSortOrder(const QModelIndex& parentIdx);

    bool findAndSelect(bool findNext = true);

    void FillClipboard(QModelIndex item, Qt::DropAction act);

    QString menuDir();

signals:
    void changed();
    void selectionChanged(const QString &);

public slots:
    void setActionsEnabled();

    bool slotDirNew();
    bool slotFileNew();
    bool slotEdit();
    void slotDel();
    void slotExe();

    void slotCut();
    void slotCopy();
    void slotPaste();

    void slotUp();
    void slotDown();

    void slotRecreate();

    void slotFind();
    void slotFindNext();
    //void slotFindPrev();

    void selectionChangedSlot();

protected:
    void contextMenuEvent(QContextMenuEvent *event) override;
    void dropEvent(QDropEvent *event) override;

private:
    void init(const QString &mPath, int iconSize, bool editable, QWidget *parent);
    void clearFindResults();

    QString     dndFormat(const  QMimeData *md) const;
    QList<QUrl> dndUrls(const  QMimeData *md, const QString &format) const;

    FlyMenuModel       *fsModel;
    FlyMenuProxyModel *proxyModel;

    QMenu   *editMenu;
    QAction *cutAct, *copyAct, *pasteAct,
            *newDirAct, *newFileAct, *editAct, *delAct, *exeAct,
            *upAct, *downAct, *recreateAct, *findAct, *findNextAct, *findPrevAct;

    bool    editable;

    QString         findString;
    QModelIndexList foundList;
    int foundListPos;
};

#endif // FLYMENUVIEW_P_H
