#ifndef FLYUIDESKTOPENTRY_H
#define FLYUIDESKTOPENTRY_H

#include <QStringList>
#include <QMap>
#include <QSharedPointer>

#include <fly/flyui_export.h>

class FlyDesktopEntry;

class FlyUIDesktopAction;

class FLYUI_EXPORT FlyUIDesktopEntry
{
public:
    FlyUIDesktopEntry(const QString& path = QString());
    virtual ~FlyUIDesktopEntry();

    bool isValid() const;

    enum SuitableError {
        Suitable = 0,
        ExeEmpty,
        ExeMalformed,
        ExeNotExists,
        ExeNotFound,
        TryExeNotExists,
        TryExeNotFound,
        UrlEmpty
    };
    bool isSuitable(SuitableError* retCode = nullptr) const;

    enum DisplayableError {
        Displayable = 0,
        NotForFly,
        Hidden,
        NoDisplay
    };
    bool isDisplayable(DisplayableError* retCode = nullptr) const;

    void setFilePath(const QString& filePath);
    QString getFilePath() const;

    bool load();
    bool save();

    void setType(const QString& type);
    QString getType() const;

    void setVersion(const QString& version);
    QString getVersion() const;

    void setName(const QString& name);
    QString getName() const;

    void setLocalName(const QString& localName);
    QString getLocalName() const;

    QString getBestAvailableName() const;

    void setGenericName(const QString& name);
    QString getGenericName() const;

    void setLocalGenericName(const QString& name);
    QString getLocalGenericName() const;

    QString getBestAvailableGenericName() const;

    void setComment(const QString& comment);
    QString getComment() const;
    void setLocalComment(const QString& localComment);
    QString getLocalComment() const;
    QString getBestAvailableComment() const;

    void setExec(const QString& exec);
    QString getExec() const;

    void setTryExec(const QString& exec);
    QString getTryExec() const;

    void setIcon(const QString& icon);
    QString getIcon() const;

    void setLocalIcon(const QString& icon);
    QString getLocalIcon() const;

    QString getBestAvailableIcon() const;

    void setURL(const QString& url);
    QString getURL() const;
    QString getAbsoluteURL() const;

    void setPath(const QString& path);
    QString getPath() const;

    void setMimeType(const QStringList& mimeTypes);
    QStringList getMimeType() const;

    void setCategories(const QStringList& categories);
    QStringList getCategories() const;

    void setKeywords(const QStringList& keywords);
    QStringList getKeywords() const;

    void setLocalKeywords(const QStringList& keywords);
    QStringList getLocalKeywords() const;

    void setOnlyShowIn(const QStringList& onlyShowIn);
    QStringList getOnlyShowIn() const;

    void setNotShowIn(const QStringList& notShowIn);
    QStringList getNotShowIn() const;

    void setTerminal(bool on);
    bool getTerminal() const;

    void setNoDisplay(bool on);
    bool getNoDisplay() const;

    void setHidden(bool on);
    bool getHidden() const;

    void setStartupNotify(bool on);
    bool getStartupNotify() const;

    void setStartupWMClass(const QString& wmClass);
    QString getStartupWMClass() const;

    void setEntry(const QString &key, const QString &value);
    QString getEntry(const QString &key) const;

    QList<FlyUIDesktopAction> getActions() const;
    void setActions(const QList<FlyUIDesktopAction>& actions);

    // deprecated keys

    void setSortOrder(const QStringList& sortOrder);
    QStringList getSortOrder() const;

    // helper methods

    void copyFrom(const FlyUIDesktopEntry& from);
    void inheritFrom(const FlyUIDesktopEntry& from);

    static QStringList getAllAppDesktopEntrySearchPaths();
    static QStringList getAllAppDesktopEntryPaths();
    static QStringList getApplicationDesktopEntryPathsForMime(const QString& mime);
    static QList<FlyUIDesktopEntry> getApplicationDesktopEntriesForMime(const QString& mime);

    //format exec string according to Desktop Entry spec
    class ExecFormatStringPrivate;
    class ExecFormatString
    {
    public:
        ExecFormatString(const QString &execFormat);

        ExecFormatString &&filePath(const QString &filePath) &&;
        ExecFormatString &&url(const QString &url) &&;
        ExecFormatString &&icon(const QString &icon) &&;
        ExecFormatString &&appName(const QString &appName) &&;
        ExecFormatString &&desktopFilePath(const QString &filePath) &&;
        ExecFormatString &&filePathList(const QStringList &filePaths) &&;
        ExecFormatString &&urlList(const QStringList &urls) &&;
        ExecFormatString &&listSep(const char sep = ' ') &&;

        operator QString() &&;

    private:
        ExecFormatStringPrivate *d;
    };

    static bool isExecSuitable(const QString& exec, SuitableError* retCode = nullptr);

private:
    void getUnknowns();
    void setUnknowns();

private:
    FlyUIDesktopEntry(FlyDesktopEntry* flyDesktopEntry);

    QSharedPointer<FlyDesktopEntry> m_desktopEntry;
    QMap<QString, QString> m_unknowns;
};

#endif // FLYUIDESKTOPENTRY_H
